@@ -1,6 +1,6 @@
 ;;; helm-buffers.el --- helm support for buffers. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2020 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2021 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 (declare-function helm-browse-project "helm-files")
 
 (defvar dired-buffers)
+(defvar org-directory)
 
 
 (defgroup helm-buffers nil
@@ -319,7 +320,8 @@ Note that this variable is buffer-local.")
    (multimatch :initform nil)
    (match :initform 'helm-buffers-match-function)
    (persistent-action :initform 'helm-buffers-list-persistent-action)
-   (keymap :initform helm-buffer-map)
+   (keymap :initform 'helm-buffer-map)
+   (find-file-target :initform #'helm-buffers-quit-and-find-file-fn)
    (migemo :initform 'nomultimatch)
    (volatile :initform t)
    (nohighlight :initform t)
@@ -429,9 +431,18 @@ The list is reordered with `helm-buffer-list-reorder-fn'."
   (append
    (list
     (concat prefix
-            (propertize buf-name 'face face1
-                        'help-echo help-echo
-                        'type type)))
+            (let* ((buf-fname (buffer-file-name (get-buffer buf-name)))
+                   (ext (if buf-fname (helm-file-name-extension buf-fname) ""))
+                   (buf-name (propertize buf-name 'face face1
+                                         'help-echo help-echo
+                                         'type type)))
+              (when (condition-case _err
+                                (string-match (format "\\.\\(%s\\)" ext) buf-name)
+                              (invalid-regexp nil))
+                        (add-face-text-property
+                         (match-beginning 1) (match-end 1)
+                         'helm-ff-file-extension nil buf-name))
+              buf-name)))
    (and details
         (list size mode
               (propertize
@@ -448,7 +459,7 @@ The list is reordered with `helm-buffer-list-reorder-fn'."
     (helm-acond ((assq major-mode helm-buffer--pretty-names)
                  (cdr it))
                 ((stringp mode-name) mode-name)
-                (t (format-mode-line mode-name)))))
+                (t (format-mode-line mode-name nil nil (get-buffer buf))))))
 
 (defun helm-buffer--details (buffer &optional details)
   (require 'dired)
@@ -1113,7 +1124,8 @@ Can be used by any source that list buffers."
   (interactive)
   (with-helm-alive-p
     (let ((filter-attrs (helm-get-attr 'filtered-candidate-transformer
-                                   helm-source-buffers-list)))
+                                   helm-source-buffers-list))
+          (sel          (helm-get-selection)))
       (if (memq 'helm-shadow-boring-buffers filter-attrs)
           (helm-set-attr 'filtered-candidate-transformer
                         (cons 'helm-skip-boring-buffers
@@ -1125,7 +1137,7 @@ Can be used by any source that list buffers."
                             (remove 'helm-skip-boring-buffers
                                     filter-attrs))
                       helm-source-buffers-list))
-      (helm-force-update))))
+      (helm-force-update (helm-buffers--quote-truncated-buffer sel)))))
 (put 'helm-buffers-toggle-show-hidden-buffers 'helm-only t)
 
 (defun helm-buffers-browse-project (buf)
@@ -1140,6 +1152,21 @@ Can be used by any source that list buffers."
       (if helm-buffers-in-project-p
           (user-error "You are already browsing this project")
           (helm-exit-and-execute-action 'helm-buffers-browse-project))))
+
+(defun helm-buffers-quit-and-find-file-fn (source)
+  (let* ((sel (helm-get-selection nil nil source))
+         (buf (helm-aand (bufferp sel)
+                         (get-buffer sel)
+                         (buffer-name it))))
+    (when buf
+      (or (buffer-file-name sel)
+          (car (rassoc buf dired-buffers))
+          (and (with-current-buffer buf
+                 (eq major-mode 'org-agenda-mode))
+               org-directory
+               (expand-file-name org-directory))
+          (with-current-buffer buf
+            (expand-file-name default-directory))))))
 
 ;;; Candidate Transformers
 ;;
