@@ -1,6 +1,6 @@
 ;;; helm-elisp.el --- Elisp symbols completion for helm. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2020 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2021 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@
 (declare-function helm-describe-face "helm-lib")
 (declare-function helm-read-file-name "helm-mode")
 (declare-function helm-comp-read "helm-mode")
-
+(declare-function helm-M-x-transformer-no-sort-no-props "helm-command")
 
 ;;; Customizable values
 
@@ -434,7 +434,7 @@ If SYM is not documented, return \"Not documented\"."
   "Preconfigured Helm to complete file name at point."
   (interactive)
   (require 'helm-mode)
-  (let* ((tap (thing-at-point 'filename))
+  (let* ((tap (or (thing-at-point 'filename) ""))
          beg
          (init (and tap
                     (or force
@@ -619,12 +619,15 @@ double quote."
                                     (customize-face (helm-symbolify candidate)))))))
 
 (defun helm-def-source--emacs-commands (&optional default)
+  (require 'helm-command)
   (helm-build-in-buffer-source "Commands"
     :init (lambda ()
             (helm-apropos-init 'commandp default))
     :fuzzy-match helm-apropos-fuzzy-match
-    :filtered-candidate-transformer (and (null helm-apropos-fuzzy-match)
-                                         'helm-apropos-default-sort-fn)
+    :filtered-candidate-transformer
+    (append (list #'helm-M-x-transformer-no-sort-no-props)
+            (and (null helm-apropos-fuzzy-match)
+                 '(helm-apropos-default-sort-fn)))
     :display-to-real 'helm-symbolify
     :nomark t
     :persistent-action (lambda (candidate)
@@ -735,14 +738,15 @@ double quote."
   "Preconfigured Helm to describe commands, functions, variables and faces.
 In non interactives calls DEFAULT argument should be provided as
 a string, i.e. the `symbol-name' of any existing symbol."
-  (interactive (list (thing-at-point 'symbol)))
-    (helm :sources
-          (mapcar (lambda (func)
-                    (funcall func default))
-                  helm-apropos-function-list)
-          :history 'helm-apropos-history
-          :buffer "*helm apropos*"
-          :preselect (and default (concat "\\_<" (regexp-quote default) "\\_>"))))
+  (interactive (list (with-syntax-table emacs-lisp-mode-syntax-table
+                       (thing-at-point 'symbol))))
+  (helm :sources
+        (mapcar (lambda (func)
+                  (funcall func default))
+                helm-apropos-function-list)
+        :history 'helm-apropos-history
+        :buffer "*helm apropos*"
+        :preselect (and default (concat "\\_<" (regexp-quote default) "\\_>"))))
 
 
 ;;; Advices
@@ -869,7 +873,7 @@ a string, i.e. the `symbol-name' of any existing symbol."
 ;;
 ;;
 (defclass helm-absolute-time-timers-class (helm-source-sync helm-type-timers)
-  ((candidates :initform timer-list)
+  ((candidates :initform 'timer-list)
    (allow-dups :initform t)
    (candidate-transformer
     :initform
@@ -881,7 +885,7 @@ a string, i.e. the `symbol-name' of any existing symbol."
   (helm-make-source "Absolute Time Timers" 'helm-absolute-time-timers-class))
 
 (defclass helm-idle-time-timers-class (helm-source-sync helm-type-timers)
-  ((candidates :initform timer-idle-list)
+  ((candidates :initform 'timer-idle-list)
    (allow-dups :initform t)
    (candidate-transformer
     :initform
