@@ -1,6 +1,6 @@
 ;;; helm-fd.el --- helm interface for fd command line tool. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2020 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2021 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -71,7 +71,7 @@
    (help-message :initform 'helm-fd-help-message)
    (filtered-candidate-transformer :initform 'helm-fd-fct)
    (action :initform 'helm-type-file-actions)
-   (keymap :initform helm-fd-map)))
+   (keymap :initform 'helm-fd-map)))
 
 (defun helm-fd-process ()
   "Initialize fd process in an helm async source."
@@ -107,7 +107,7 @@
 (defun helm-fd-fct (candidates _source)
   "The filtered-candidate-transformer function for helm-fd."
   (cl-loop for i in candidates
-           collect (helm--ansi-color-apply i)))
+           collect (ansi-color-apply i)))
 
 (defun helm-fd-1 (directory)
   "Run fd shell command on DIRECTORY with helm interface."
