@@ -1,6 +1,6 @@
 ;;; helm-grep.el --- Helm Incremental Grep. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2020 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2021 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -30,7 +30,6 @@
 (declare-function doc-view-goto-page "doc-view" (page))
 (declare-function pdf-view-goto-page "pdf-view" (page &optional window))
 (declare-function helm-mm-split-pattern "helm-multi-match")
-(declare-function helm--ansi-color-apply "helm-lib")
 (declare-function helm-comp-read "helm-mode")
 (declare-function helm-occur "helm-occur")
 
@@ -75,8 +74,10 @@ don't specify the '%e' format spec.
 Helm also support ack-grep and git-grep.  The following is a
 default command example for ack-grep:
 
-\(setq helm-grep-default-command \"ack-grep -Hn --color --smart-case --no-group %e %p %f\"
-       helm-grep-default-recurse-command \"ack-grep -H --color --smart-case --no-group %e %p %f\")
+\(setq helm-grep-default-command
+       \"ack-grep -Hn --color --smart-case --no-group %e -- %p %f\"
+       helm-grep-default-recurse-command
+       \"ack-grep -H --color --smart-case --no-group %e -- %p %f\")
 
 You can ommit the %e spec if you don't want to be prompted for
 types.
@@ -242,7 +243,17 @@ doing."
   "A list of additional parameters to pass to grep-ag pipe command.
 Use parameters compatibles with the backend you are using
 \(i.e. AG for AG, PT for PT or RG for RG)
+Here are the commands where you may want to add switches:
 
+    ag -S --color
+    rg -N -S --color=?
+
+For RG the value of --color= is computed according to the --color=
+value used in `helm-grep-ag-command'.
+
+Note also that by default the \"--\" option is always used, you don't
+need to add it here.
+ 
 You probably don't need to use this unless you know what you are
 doing."
   :group 'helm-grep
@@ -787,6 +798,13 @@ If N is positive go forward otherwise go backward."
     (helm-exit-and-execute-action 'helm-grep-save-results)))
 (put 'helm-grep-run-save-buffer 'helm-only t)
 
+(defun helm-grep-quit-an-find-file-fn (source)
+  (let* ((sel (helm-get-selection nil nil source))
+         (grep-line (and (stringp sel)
+                         (helm-grep-split-line sel))))
+    (if (and grep-line (file-exists-p (car grep-line)))
+        (expand-file-name (car grep-line))
+      default-directory)))
 
 ;;; helm-grep-mode
 ;;
@@ -891,7 +909,7 @@ Special commands:
                               (prog1 (split-string (buffer-string) "\n")
                                 (kill-buffer)))
                    for line = (if (string-match-p helm--ansi-color-regexp l)
-                                  (helm--ansi-color-apply l) l)
+                                  (ansi-color-apply l) l)
                    when (string-match helm-grep-split-line-regexp line)
                    do (insert (propertize
                                (car (helm-grep-filter-one-by-one
@@ -1049,7 +1067,7 @@ of grep."
 (defclass helm-grep-class (helm-source-async)
   ((candidates-process :initform 'helm-grep-collect-candidates)
    (filtered-candidate-transformer :initform #'helm-grep-fc-transformer)
-   (keymap :initform helm-grep-map)
+   (keymap :initform 'helm-grep-map)
    (pcre :initarg :pcre :initform nil
          :documentation
          "  Backend is using pcre regexp engine when non-nil.")
@@ -1071,6 +1089,7 @@ of grep."
    (requires-pattern :initform 2)
    (before-init-hook :initform 'helm-grep-before-init-hook)
    (after-init-hook :initform 'helm-grep-after-init-hook)
+   (find-file-target :initform #'helm-grep-quit-an-find-file-fn)
    (group :initform 'helm-grep)))
 
 (defvar helm-source-grep nil)
@@ -1214,7 +1233,7 @@ matching `helm-zgrep-file-extension-regexp' only."
   (let* ((root   (or dir (and helm-grep-default-directory-fn
                               (funcall helm-grep-default-directory-fn))))
          (ansi-p (string-match-p helm--ansi-color-regexp candidate))
-         (line   (if ansi-p (helm--ansi-color-apply candidate) candidate))
+         (line   (if ansi-p (ansi-color-apply candidate) candidate))
          (split  (helm-grep-split-line line))
          (fname  (if (and root split)
                      ;; Filename should always be provided as a local
@@ -1446,7 +1465,7 @@ non-file buffers."
 ;;  https://github.com/BurntSushi/ripgrep
 
 (defcustom helm-grep-ag-command
-  "ag --line-numbers -S --color --nogroup %s %s %s"
+  "ag --line-numbers -S --color --nogroup %s -- %s %s"
   "The default command for AG, PT or RG.
 
 Takes three format specs, the first for type(s), the second for
@@ -1461,12 +1480,12 @@ colorization of backend, however it is still supported.
 
 For ripgrep here is the command line to use:
 
-    rg --color=always --smart-case --no-heading --line-number %s %s %s
+    rg --color=always --smart-case --no-heading --line-number %s -- %s %s
 
 And to customize colors (always for ripgrep) use something like this:
 
     rg --color=always --colors 'match:bg:yellow' --colors 'match:fg:black'
-\--smart-case --no-heading --line-number %s %s %s
+\--smart-case --no-heading --line-number %s -- %s %s
 
 This will change color for matched items from foreground red (the
 default) to a yellow background with a black foreground.  Note
@@ -1538,7 +1557,7 @@ returns if available with current AG version."
                       (shell-quote-argument directory))))
     (helm-aif (cdr patterns)
         (concat cmd (cl-loop for p in it concat
-                             (format " | %s %s"
+                             (format " | %s -- %s"
                                      pipe-cmd (shell-quote-argument p))))
       cmd)))
 
@@ -1610,7 +1629,7 @@ returns if available with current AG version."
    (pcre :initarg :pcre :initform t
          :documentation
          "  Backend is using pcre regexp engine when non--nil.")
-   (keymap :initform helm-grep-map)
+   (keymap :initform 'helm-grep-map)
    (history :initform 'helm-grep-ag-history)
    (help-message :initform 'helm-grep-help-message)
    (filtered-candidate-transformer :initform #'helm-grep-fc-transformer)
@@ -1620,6 +1639,7 @@ returns if available with current AG version."
    (requires-pattern :initform 2)
    (nomark :initform t)
    (action :initform 'helm-grep-actions)
+   (find-file-target :initform #'helm-grep-quit-an-find-file-fn)
    (group :initform 'helm-grep)))
 
 (defvar helm-source-grep-ag nil)
@@ -1631,8 +1651,9 @@ returns if available with current AG version."
                  (assoc-default 'follow helm-source-grep-ag))
       (setf (slot-value source 'follow) it)))
 
-(defun helm-grep-ag-1 (directory &optional type)
-  "Start helm ag in DIRECTORY maybe searching in files of type TYPE."
+(defun helm-grep-ag-1 (directory &optional type input)
+  "Start helm ag in DIRECTORY maybe searching in files of type TYPE.
+If INPUT is provided, use it as the search string."
   (setq helm-source-grep-ag
         (helm-make-source (upcase (helm-grep--ag-command)) 'helm-grep-ag-class
           :header-name (lambda (name)
@@ -1644,6 +1665,7 @@ returns if available with current AG version."
   (helm :sources 'helm-source-grep-ag
         :keymap helm-grep-map
         :history 'helm-grep-ag-history
+        :input input
         :truncate-lines helm-grep-truncate-lines
         :buffer (format "*helm %s*" (helm-grep--ag-command))))
 
