@@ -1,6 +1,6 @@
 ;;; helm-help.el --- Help messages for Helm. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2020 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2021 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -433,6 +433,16 @@ TIP: Use `\\<helm-map>\\[helm-toggle-resplit-and-swap-windows]' and `\\[helm-enl
 and to enlarge it while viewing images.
 Note this may not work with exotic Helm windows settings such as the ones in Spacemacs.
 
+**** Show thumbnails
+
+Helm use image-dired to show thumbnails on image files, you can
+toggle the thumbnail view with \\<helm-find-files-map>`\\[helm-ff-toggle-thumbnails]'.
+
+**** Launch a slideshow from marked files
+
+Helm provides an action from `helm-find-files' that allows running a slideshow on marked files.
+Just mark image files and launch slideshow from action menu, bindings are self documented.
+
 *** Open files externally
 
 - Open file with external program (`\\<helm-find-files-map>\\[helm-ff-run-open-file-externally]',`C-u' to choose).
@@ -446,6 +456,10 @@ prefix arg Helm will see if there is an application suitable in
 mailcap files.  If you want to specify which external application
 to use (and its options) use a prefix arg.
 
+If you have to pass arguments after filename use `%s' in your command e.g. \"foo %s -a -b\"
+If you want to detach your program from Emacs, you can use e.g. \"(foo %s &)\" (only supported on Linux/Unix).
+When using `%s' do not quote it (i.e. \"%s\"), helm is already quoting filename argument.
+
 Note: What you configure for Helm in `helm-external-programs-associations'
 will take precedence on mailcap files.
 
@@ -455,8 +469,7 @@ Same as above but doesn't quit Helm session, it is apersistent action.
 
 - Open file externally with default tool (`\\[helm-ff-run-open-file-with-default-tool]').
 
-This uses xdg-open which sucks most of the time, but perhaps it
-works fine on Windows.  This is why it is kept in Helm.
+Use `xdg-open' to open files.
 
 *** Toggle auto-completion
 
@@ -475,11 +488,7 @@ Simply write the path in the prompt and press `RET', e.g.
 
 *** To create a new file, enter a filename not ending with \"/\"
 
-Note that when you enter a new name, this one is prefixed with
-\[?] if you are in a writable directory.  If you are in a directory
-where you have no write permission the new file name is not
-prefixed and is colored in red.  There is not such distinction
-when using Tramp, new filename just appears on top of buffer.
+Note that when you enter a new name, this one is prefixed with [?].
 
 *** Recursive search from Helm-find-files
 
@@ -841,6 +850,19 @@ automatically \"-e 'ssh -p 2222'\" to the rsync command line
 unless you have specified yourself the \"-e\" option by editing
 rsync command line with a prefix arg (see above).
 
+*** Access files on Android phones from Helm
+
+Since Android doesn't provide anymore mass storage for USB, it is
+not simple to access files on Android, the best way to do this
+actually seems to use Adb, here some hints to set this up, read
+in addition the Tramp documentation.
+
+1) Install Adb, most distribution provide it.
+2) Enable on your phone USB debug in System/dvlpmnt settings.
+3) From helm-find-files use adb tramp method:
+    /adb::/
+From there you can navigate as usual, mark and copy files etc...
+
 *** Bookmark the `helm-find-files' session
 
 You can bookmark the `helm-find-files' session with `\\[helm-ff-bookmark-set]'.
@@ -911,6 +933,12 @@ $ example foo bar baz
 
 Of course the alias command should support this.
 
+NOTE: Helm assume that any alias command ending with '$*' or
+'$*&' supports many files as arguments, so no need to give a
+prefix arg for such alias, however if your command is not an
+alias you have to specify a prefix arg if you want your command
+to apply all files at once.
+
 If you add %s to the command line %s will be replaced with the candidate, this mean you can
 add extra argument to your command e.g. command -extra-arg %s or command %s -extra-arg.
 If you want to pass many files inside %s, don't forget to use a prefix arg.
@@ -949,6 +977,7 @@ the same hosts.
 As a rule of thumb, prefer the scp method unless using multihops (which only
 works with the ssh method), especially when copying large files.
 
+IMPORTANT:
 You need to hit `C-j' once on top of a directory on the first connection
 to complete the pattern in the minibuffer.
 
@@ -971,6 +1000,11 @@ without addind the ending \":\", second hit insert the last \":\".
 As soon the last \":\" is entered TRAMP will kick in and you should see the list
 of candidates soon after.
 
+**** Completion on tramp methods
+
+If you enter \":\" directly after \"/\" or \"|\" you will have completion on tramp methods,
+hitting `\\[helm-execute-persistent-action]' or `right' on a method will insert it in minibuffer.
+ 
 When connection fails, be sure to delete your TRAMP connection with M-x
 `helm-delete-tramp-connection' before retrying.
 
@@ -1174,6 +1208,7 @@ you will have to refresh manually directories when needed with `\\<helm-map>\\[h
 |\\[helm-ff-rotate-right-persistent]|Rotate image right.
 |\\[helm-ff-increase-image-size-persistent]|Zoom in image.
 |\\[helm-ff-decrease-image-size-persistent]|Zoom out image.
+|\\[helm-ff-toggle-thumbnails]|Show thumbnails on image files.
 |\\[helm-find-files-up-one-level]|Go to parent directory.
 |\\[helm-find-files-history]|Switch to the visited-directory history.
 |\\[helm-ff-file-name-history]|Switch to file name history.
@@ -1191,11 +1226,12 @@ you will have to refresh manually directories when needed with `\\<helm-map>\\[h
 |\\[helm-ff-run-insert-org-link]|Insert org link.
 |\\[helm-ff-bookmark-set]|Set bookmark to current directory.
 |\\[helm-find-files-switch-to-bookmark]|Jump to bookmark list.
-|\\[helm-ff-sort-alpha]|Sort alphabetically
-|\\[helm-ff-sort-by-newest]|Sort by newest
-|\\[helm-ff-sort-by-size]|Sort by size
-|\\[helm-ff-toggle-dirs-only]|Show only directories
-|\\[helm-ff-toggle-files-only]|Show only files")
+|\\[helm-ff-sort-alpha]|Sort alphabetically.
+|\\[helm-ff-sort-by-newest]|Sort by newest.
+|\\[helm-ff-sort-by-size]|Sort by size.
+|\\[helm-ff-toggle-dirs-only]|Show only directories.
+|\\[helm-ff-toggle-files-only]|Show only files.
+|\\[helm-ff-sort-by-ext]|Sort by extensions.")
 
 ;;; Help for file-name-history
 ;;
@@ -1950,23 +1986,37 @@ You can of course combine both placeholders if needed.
 
 *** Specify marked files as arguments
 
+When you have marked files and your command support only one file
+as arg, helm will execute command sequencially on each file like
+this:
+
 Example:
 
-    <command> file1 file2...
-
-Call `helm-find-files-eshell-command-on-file' with one prefix argument.  Otherwise
-you can pass one prefix argument from the command selection buffer.
-
-Note: This does not work on remote files.
-
-With two prefix-args the output is printed to the `current-buffer'.
-
-With no prefix argument or a prefix argument value of '(16) (`\\[universal-argument] \\[universal-argument]')
-the command is called once for each file like this:
-
     <command> file1
     <command> file2
-    ...
+    ...etc
+
+When you have marked files and your command accept many files at
+once helm will run your command with all files at once like this:
+
+Example:
+
+    <command> file1 file2 etc...
+
+The two use case above are applied automatically by Helm
+depending if your command is an eshell alias which value ends by
+'$1' or '$*'.  If your command is not an alias, i.e. you entered
+an arbitrary command on prompt with '%s' to specify filenames,
+you will have to pass one prefix argument from the command
+selection buffer.
+
+Note: This does not work on remote files.
+
+With two prefix-args the output is printed to the
+`current-buffer', the command being executed in the same
+conditions as described above.
+NOTE: If your command is not an alias, you can't pass all files at once and print in current buffer at the same time.
+Also note that running multiple files at once is not supported with remote files.
 
 *** Run eshell commands asynchronously
 
@@ -1974,10 +2024,6 @@ You can run your commands asynchronously by adding \"&\" at end
 of any commands, e.g. \"foo %s &\".  You can also directly setup
 your alias in the eshell alias file with e.g. \"alias foo $1 &\".
 
-NOTE: If you use \"&\" in a command with marked files and your
-command accept many files as argument don't forget to pass the
-prefix arg to ensure you run only one command on all marked async.
-
 ** Commands
 \\<helm-esh-on-file-map>")
 
