@@ -1,6 +1,6 @@
 ;;; helm-imenu.el --- Helm interface for Imenu -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2020 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2021 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -153,9 +153,10 @@ string."
    (persistent-action :initform 'helm-imenu-persistent-action)
    (persistent-help :initform "Show this entry")
    (nomark :initform t)
-   (keymap :initform helm-imenu-map)
+   (keymap :initform 'helm-imenu-map)
    (help-message :initform 'helm-imenu-help-message)
    (action :initform 'helm-imenu-action)
+   (find-file-target :initform #'helm-imenu-quit-and-find-file-fn)
    (group :initform 'helm-imenu)))
 
 (defcustom helm-imenu-fuzzy-match nil
@@ -189,6 +190,11 @@ string."
           (helm-force-update))
         t)))
 
+(defun helm-imenu-quit-and-find-file-fn (source)
+  (let ((sel (helm-get-selection nil nil source)))
+    (when (and (consp sel) (markerp (cdr sel)))
+      (buffer-file-name (marker-buffer (cdr sel))))))
+
 (defun helm-imenu-action (candidate)
   "Default action for `helm-source-imenu'."
   (helm-log-run-hook 'helm-goto-line-before-hook)
