@@ -1,6 +1,6 @@
 ;;; helm-net.el --- helm browse url and search web. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2020 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2021 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -262,22 +262,24 @@ Can be \"-new-tab\" (default) or \"-new-window\"."
 (defvar helm-browse-url-nyxt-program "nyxt")
 (defvar helm-browse-url-conkeror-program "conkeror")
 (defvar helm-browse-url-opera-program "opera")
+(defvar helm-browse-url-w3m-program (or (and (boundp 'w3m-command) w3m-command)
+                                        (executable-find "w3m")))
 (defvar helm-browse-url-default-browser-alist
-  `((,(or (and (boundp 'w3m-command) w3m-command)
-          "/usr/bin/w3m") . w3m-browse-url)
-    (,browse-url-firefox-program . browse-url-firefox)
-    (,helm-browse-url-chromium-program . helm-browse-url-chromium)
-    (,helm-browse-url-conkeror-program . helm-browse-url-conkeror)
-    (,helm-browse-url-opera-program . helm-browse-url-opera)
-    (,helm-browse-url-uzbl-program . helm-browse-url-uzbl)
-    (,browse-url-kde-program . browse-url-kde)
-    (,browse-url-gnome-moz-program . browse-url-gnome-moz)
-    (,browse-url-mozilla-program . browse-url-mozilla)
-    (,browse-url-galeon-program . browse-url-galeon)
-    (,browse-url-netscape-program . browse-url-netscape)
-    (,browse-url-xterm-program . browse-url-text-xterm)
+  '((helm-browse-url-w3m-program . w3m-browse-url)
+    (browse-url-firefox-program . browse-url-firefox)
+    (helm-browse-url-chromium-program . helm-browse-url-chromium)
+    (helm-browse-url-conkeror-program . helm-browse-url-conkeror)
+    (helm-browse-url-opera-program . helm-browse-url-opera)
+    (helm-browse-url-uzbl-program . helm-browse-url-uzbl)
+    (helm-browse-url-nyxt-program . helm-browse-url-nyxt)
+    (browse-url-kde-program . browse-url-kde)
+    (browse-url-gnome-moz-program . browse-url-gnome-moz)
+    (browse-url-mozilla-program . browse-url-mozilla)
+    (browse-url-galeon-program . browse-url-galeon)
+    (browse-url-netscape-program . browse-url-netscape)
+    (browse-url-xterm-program . browse-url-text-xterm)
     ("emacs" . eww-browse-url))
-  "*Alist of \(executable . function\) to try to find a suitable url browser.")
+  "Alist of (browse_url_variable . function) to try to find a suitable url browser.")
 
 (cl-defun helm-generic-browser (url cmd-name &rest args)
   "Browse URL with NAME browser."
@@ -346,15 +348,18 @@ NOTE: Probably not supported on some systems (e.g., Windows)."
 
 ;;;###autoload
 (defun helm-browse-url-nyxt (url &optional _ignore)
-  "Browse URL with next browser."
+  "Browse URL with nyxt browser."
   (interactive "sURL: ")
   (helm-generic-browser url helm-browse-url-nyxt-program))
 
 (defun helm-browse-url-default-browser (url &rest args)
   "Find the first available browser and ask it to load URL."
   (let ((default-browser-fn
-         (cl-loop for (exe . fn) in helm-browse-url-default-browser-alist
-               thereis (and exe (executable-find exe) (fboundp fn) fn))))
+         (cl-loop for (var . fn) in helm-browse-url-default-browser-alist
+                  for exe = (if (stringp var)
+                                var
+                              (and (boundp var) (symbol-value var)))
+                  thereis (and exe (executable-find exe) (fboundp fn) fn))))
     (if default-browser-fn
         (apply default-browser-fn url args)
       (error "No usable browser found"))))
@@ -402,8 +407,7 @@ NOTE: Probably not supported on some systems (e.g., Windows)."
      (helm-build-elvi-list)
      :must-match t
      :name "Surfraw Search Engines"
-     :del-input nil
-     :history helm-surfraw-engines-history)))
+     :history 'helm-surfraw-engines-history)))
   (let* ((engine-nodesc (car (split-string engine)))
          (url (if (string= engine-nodesc "duckduckgo")
                   ;; "sr duckduckgo -p foo" is broken, workaround.
@@ -418,9 +422,7 @@ NOTE: Probably not supported on some systems (e.g., Windows)."
                                           browse-url-browser-function)))
     (if (string= engine-nodesc "W")
         (helm-browse-url helm-home-url)
-      (helm-browse-url url)
-      (setq helm-surfraw-engines-history
-            (cons engine (delete engine helm-surfraw-engines-history))))))
+      (helm-browse-url url))))
 
 ;;;###autoload
 (defun helm-google-suggest ()
