@@ -148,7 +148,7 @@
 
    (keymap
     :initarg :keymap
-    :initform helm-map
+    :initform 'helm-map
     :custom sexp
     :documentation
     "  Specific keymap for this source.
@@ -684,9 +684,17 @@
     minibuffer contents as candidate value (in this case helm-buffer
     is empty).")
 
+   (find-file-target
+    :initarg :find-file-target
+    :initform nil
+    :custom function
+    :documentation
+    "  Determine the target file when running `helm-quit-and-find-file'.
+  It is a function called with one arg SOURCE.")
+
    (group
     :initarg :group
-    :initform helm
+    :initform 'helm
     :custom symbol
     :documentation
     "  The current source group, default to `helm' when not specified."))
