@@ -1,6 +1,6 @@
 ;;; helm-utils.el --- Utilities Functions for helm. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2020 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2021 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -579,71 +579,42 @@ from its directory."
     (require 'helm-grep)
     (require 'helm-elisp)
     (require 'bookmark) ; For bookmark-alist
-    (helm-run-after-exit
-     (lambda (f)
-       ;; Ensure specifics `helm-execute-action-at-once-if-one'
-       ;; fns don't run here.
-       (let (helm-execute-action-at-once-if-one
-             helm-actions-inherit-frame-settings) ; use this-command
-         (if (file-exists-p f)
-             (helm-find-files-1 (file-name-directory f)
-                                (concat
-                                 "^"
-                                 (regexp-quote
-                                  (if helm-ff-transformer-show-only-basename
-                                      (helm-basename f) f))))
+    (let ((src (helm-get-current-source)))
+      (helm-run-after-exit
+       (lambda (f)
+         ;; Ensure specifics `helm-execute-action-at-once-if-one'
+         ;; fns don't run here.
+         (let (helm-execute-action-at-once-if-one
+               helm-actions-inherit-frame-settings) ; use this-command
+           (if (file-exists-p f)
+               (helm-find-files-1 (file-name-directory f)
+                                  (concat
+                                   "^"
+                                   (regexp-quote
+                                    (if helm-ff-transformer-show-only-basename
+                                        (helm-basename f) f))))
              (helm-find-files-1 f))))
-     (let* ((sel       (helm-get-selection))
-            (marker    (and (consp sel) (markerp (cdr sel))))
-            (grep-line (and (stringp sel)
-                            (helm-grep-split-line sel)))
-            (occur-fname (helm-aand (numberp sel)
-                                    (helm-get-attr 'buffer-name)
-                                    (buffer-file-name (get-buffer it))))
-            (bmk-name  (and (stringp sel)
-                            (not grep-line)
-                            (replace-regexp-in-string "\\`\\*" "" sel)))
-            (bmk       (and bmk-name (assoc bmk-name bookmark-alist)))
-            (buf       (helm-aif (and (bufferp sel) (get-buffer sel))
-                           (buffer-name it)))
-            (pkg       (and (stringp sel)
-                            (get-text-property 0 'tabulated-list-id sel)))
-            (default-preselection (or (buffer-file-name helm-current-buffer)
-                                      default-directory)))
-       (cond
-         ;; Buffer.
-         (buf (or (buffer-file-name sel)
-                  (car (rassoc buf dired-buffers))
-                  (and (with-current-buffer buf
-                         (eq major-mode 'org-agenda-mode))
-                       org-directory
-                       (expand-file-name org-directory))
-                  (with-current-buffer buf
-                    (expand-file-name default-directory))))
-         ;; imenu (marker).
-         (marker
-          (or (buffer-file-name (marker-buffer (cdr sel)))
-              default-preselection))
-         ;; Bookmark.
-         (bmk (helm-aif (bookmark-get-filename bmk)
-                  (if (and helm--url-regexp
-                           (string-match helm--url-regexp it))
-                      it (expand-file-name it))
-                (expand-file-name default-directory)))
+       (helm--quit-and-find-file-default-file src)))))
+(put 'helm-quit-and-find-file 'helm-only t)
+
+(defun helm--quit-and-find-file-default-file (source)
+  (let ((target-fn (helm-get-attr 'find-file-target)))
+    ;; target-fn function may return nil, in this case fallback to default.
+    (helm-aif (and target-fn (funcall target-fn source))
+        it
+      (let* ((sel                  (helm-get-selection nil nil source))
+             (default-preselection (or (helm-default-directory)
+                                       (buffer-file-name helm-current-buffer)
+                                       default-directory)))
+        (cond
          ((and (stringp sel) (or (file-remote-p sel)
                                  (file-exists-p sel)))
           (expand-file-name sel))
-         ;; Grep.
-         ((and grep-line (file-exists-p (car grep-line)))
-          (expand-file-name (car grep-line)))
-         ;; Occur.
-         ((and occur-fname (file-exists-p occur-fname))
-          (expand-file-name occur-fname))
-         ;; Package (installed).
-         ((and pkg (package-installed-p pkg))
-          (expand-file-name (package-desc-dir pkg)))
          ;; Url.
-         ((and (stringp sel) helm--url-regexp (string-match helm--url-regexp sel)) sel)
+         ((and (stringp sel)
+               helm--url-regexp
+               (string-match helm--url-regexp sel))
+          sel)
          ;; Exit brutally from a `with-helm-show-completion'
          ((and helm-show-completion-overlay
                (overlayp helm-show-completion-overlay))
@@ -652,7 +623,6 @@ from its directory."
           (expand-file-name default-preselection))
          ;; Default.
          (t (expand-file-name default-preselection)))))))
-(put 'helm-quit-and-find-file 'helm-only t)
 
 (defun helm-generic-sort-fn (s1 s2)
   "Sort predicate function for helm candidates.
