@@ -1,6 +1,6 @@
 ;;; helm-x-files.el --- helm auxiliary functions and sources. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2020 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2021 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -56,8 +56,8 @@
                               (or (string-match helm-tramp-file-name-regexp f)
                                   (file-exists-p f)))
                             (mapcar 'car session-file-alist))))
-   (keymap       :initform helm-generic-files-map)
-   (help-message :initform helm-generic-file-help-message)
+   (keymap       :initform 'helm-generic-files-map)
+   (help-message :initform 'helm-generic-file-help-message)
    (action       :initform 'helm-type-file-actions)))
 
 (defvar helm-source-session nil
